==> libmatrix-0.04.tbz: libmatrix-0.04/vector.c <==
/*
vector.c

matrix manipulation library
version 0.04
Peter A. Cejchan, GPL
Mon Jul 31 12:24:25 MET 2006
<cej@gli.cas.cz | pac7@post.cz>

float vector structure and functions

*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "libmatrix.h" 
#include "vector.h" 


// functions:

/*** old version ***
int 
vec_read(Vector *vec)
// read whitespace-delimited vector
{
	char *buf, *av[MAXLENGTH];
	int j;

	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		vec->len=   tokenize(buf, av, MAXLENGTH);
		for(j=0; j<vec->len; j++){
			vec->x[j]=atof(av[j]);
		}
		free(buf);
	}
	return 1;
}
*/

int 
vec_read(Vector *V)
// read whitespace-delimited vector
{
	char *buf, *av[MAXCOLS];
	int i, n;
	for(i=0; i<V->len; i++){
		if((buf = Brdstr(&bin, '\n', 1)) != nil){	
			// skip comment lines
			while((buf[0]=='#') || (buf[0]=='>')){

//				print("comments\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
			}	
			n=tokenize(buf, av, V->len);
			// skip whitespace lines
			while(n==0){
				print("whitespace\n");	
//				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
				n=tokenize(buf, av, V->len);
			}	
			V->x[i]=atof(av[i]);
			free(buf);
		}
		else return 0;
	}
	return 1;
}

Vector *
vec_alloc(int len)
// allocates memory for a vector
{
	int i;
	Vector *V;

	V=(Vector*)malloc(sizeof(Vector));
	V->len=len; 
	V->x=(double*)malloc(V->len*sizeof(double));
	for(i=0;i<V->len;i++)
		V->x[i]=0.0;
	return V;
}


void 
vec_write(Vector *V)
// write out a vector
{
	int i;
	print("%d ", V->len);
	for(i=0; i<V->len; i++){
		print("%f ", V->x[i]);
	}
	print("\n");
	return;
}


int 
vec_max(Vector *V)
//prints out col index of first occurrence of max value
{
	int i, maxi=0;
	double maxx;
	maxx = -MAXFLOAT;
	for(i=0; i<V->len; i++){
		if(V->x[i] > maxx) {
			maxx = V->x[i];
			maxi = i;
		}
	}
	print("%d\n", maxi);
	return 1;
}

double 
vec_scalar_prod(Vector *v1,Vector *v2)
// SUM[ v1i*v2i ]
{
  int i;
  double sum = 0;

  for(i=0; i<v1->len; i++)
     sum += v1->x[i]*v2->x[i];
  return sum;
}

==> libmatrix-0.05.tbz: libmatrix-0.05/vector.c <==
/*
vector.c

matrix manipulation library
version 0.05
Peter A. Cejchan, GPL
Mon Jul 31 12:24:25 MET 2006
<cej@gli.cas.cz | pac7@post.cz>

float vector structure and functions

*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "libmatrix.h" 
#include "vector.h" 


// functions:

/*** old version ***
int 
vec_read(Vector *vec)
// read whitespace-delimited vector
{
	char *buf, *av[MAXLENGTH];
	int j;

	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		vec->len=   tokenize(buf, av, MAXLENGTH);
		for(j=0; j<vec->len; j++){
			vec->x[j]=atof(av[j]);
		}
		free(buf);
	}
	return 1;
}
*/

int 
vec_read(Vector *V)
// read whitespace-delimited vector
{
	char *buf, *av[MAXCOLS], first;
	int i, n;

	n=0;
	while(n==0){
		if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;			// nothing to read
		first=buf[0];
				print("first==%c\n", first);
		n=tokenize(buf, av, V->len);
		if(n!=0){
			// skip comment lines
			if((first=='#') || (first=='>')){
				print("comments\n");
				n=0;
			}
		}
		else{
		// skip whitespace lines
			print("whitespace\n");	
		}
	}	// ends up with n > 0
				n=tokenize(buf, av, V->len);
			print("len=%d\n", V->len);	
			for(i=0; i<V->len; i++){
				V->x[i]=atof(av[i]);
print("xi=%f\n", V->x[i]);	

			}
			free(buf);
	return 1;
}

Vector *
vec_alloc(int len)
// allocates memory for a vector
{
	int i;
	Vector *V;

	V=(Vector*)malloc(sizeof(Vector));
	V->len=len; 
	V->x=(double*)malloc(V->len*sizeof(double));
	for(i=0;i<V->len;i++)
		V->x[i]=0.0;
	return V;
}


void 
vec_write(Vector *V)
// write out a vector
{
	int i;
	print("# length\n%d\n\n", V->len);
	print("# data\n");

	for(i=0; i<V->len; i++){
		print("%f ", V->x[i]);
	}
	print("\n");
	return;
}


int 
vec_max(Vector *V, double *maxx)
//returns max value found
{
	int i, ret=0;
//	double maxx;
	*maxx = -MAXFLOAT;
	for(i=0; i<V->len; i++){
		if(V->x[i] > *maxx) {
			*maxx = V->x[i];
			ret=1;
		}
	}
//	print("%f\n", *maxx);
	return ret;
}

int 
vec_wheremax(Vector *V, int *col)
//prints out col index of first occurrence of max value
{
	int i, ret=0;
	double maxx;
	maxx = -MAXFLOAT;
	*col=0;
	for(i=0; i<V->len; i++){
		if(V->x[i] > maxx) {
			maxx = V->x[i];
			*col = i;
			ret=1;
		}
	}
//	print("%d\n", *col);
	return ret;
}

double 
vec_scalar_prod(Vector *v1,Vector *v2)
// SUM[ v1i*v2i ]
{
  int i;
  double sum = 0;

  for(i=0; i<v1->len; i++)
     sum += v1->x[i]*v2->x[i];
  return sum;
}
